@@ -9,20 +9,11 @@
 #include <libmpd/libmpdclient.h>
 #include "gimmix-config.h"
 
-/* the SongInfo structure */
-typedef struct songinfo
-{
-	char *title;
-	char *artist;
-	char *album;
-	char *genre;
-	char *file;
-} SongInfo;
-
 typedef enum { 	PLAY = 1,
-				PAUSE,
-				STOP,
-				UNKNOWN,
+		PAUSE,
+		STOP,
+		UNKNOWN,
+		ERROR
 } GimmixStatus;
 
 /* create a mpd object and connect to mpd using the conf */
@@ -39,12 +30,6 @@ bool gimmix_seek (MpdObj *, int);
 /* get full image path (returned string should be freed) */
 char *gimmix_get_full_image_path (const char *);
 
-/* Gets the information of currently playing song (artist, title, genre.etc)*/
-SongInfo * gimmix_get_song_info (MpdObj *);
-
-/* Free memory allocated by gimmix_get_song_info() */
-void gimmix_free_song_info (SongInfo *);
-
 /* Get the "elapsed time / total time" as a string */
 void gimmix_get_progress_status (MpdObj *, float *, char *);
 
@@ -61,4 +46,7 @@ bool is_gimmix_shuffle (MpdObj *);
 /* strips file extension */
 void gimmix_strip_file_ext (char *string);
 
+/* replace character c1 by c2 in string */
+void gimmix_strcrep (char *string, char c1, char c2);
+
 #endif
