@@ -1,7 +1,7 @@
 /*
  * gimmix-interface.c
  *
- * Copyright (C) 2006-2007 Priyank Gosalia
+ * Copyright (C) 2006-2008 Priyank Gosalia
  *
  * Gimmix is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -28,6 +28,7 @@
 #include "gimmix-tooltip.h"
 #include "gimmix-playlist.h"
 #include "gimmix-tagedit.h"
+#include "gimmix-metadata.h"
 #include "gimmix-prefs.h"
 #include "gimmix.h"
 
@@ -35,6 +36,14 @@
 #	include "config.h"
 #endif
 
+#ifdef HAVE_LYRICS
+#	include "gimmix-lyrics.h"
+#endif
+
+#ifdef HAVE_COVER_PLUGIN
+#	include "gimmix-covers.h"
+#endif
+
 #define GIMMIX_APP_ICON  	"gimmix.png"
 
 gint 			status;
@@ -42,8 +51,11 @@ GtkWidget		*main_window;
 GtkWidget 		*progress = NULL;
 GtkWidget		*shuffle_toggle_button;
 GtkWidget		*repeat_toggle_button;
+GtkWidget		*volume_window;
 GtkWidget		*volume_scale;
 GtkWidget		*volume_button;
+GtkWidget		*volume_hscalebox;
+GtkWidget		*volume_hscale;
 GtkWidget		*song_label;
 GtkWidget		*artist_label;
 GtkWidget		*search_entry;
@@ -51,24 +63,33 @@ GtkWidget		*playlist_button;
 GtkWidget		*playlist_box;
 GtkWidget		*image_play;
 GtkWidget		*play_button;
+GtkWidget		*prev_button;
+GtkWidget		*info_button;
+GtkWidget		*next_button;
+GtkWidget		*pref_button;
+GtkWidget		*stop_button;
+GtkWidget		*plcontrolshbox;
 GtkTooltips 		*play_button_tooltip = NULL;
 
-extern MpdObj 			*gmo;
-extern GladeXML 		*xml;
-extern ConfigFile		conf;
-extern GimmixTooltip 		*tooltip;
+extern MpdObj 		*gmo;
+extern GladeXML 	*xml;
+extern ConfigFile	conf;
+extern GimmixTooltip 	*tooltip;
+extern GtkWidget	*current_playlist_treeview;
 
-extern GtkWidget		*current_playlist_treeview;
+mpd_Song		*glob_song_info;
 
-static void			gimmix_update_volume (void);
-static void			gimmix_update_repeat (void);
-static void			gimmix_update_shuffle (void);
+static void		gimmix_update_volume (void);
+static void		gimmix_update_repeat (void);
+static void		gimmix_update_shuffle (void);
+static gboolean		inited = FALSE;
 
-static gboolean			is_user_searching (void);
-static gboolean 		gimmix_timer (void);
+static gboolean		is_user_searching (void);
+static gboolean 	gimmix_timer (void);
 
 /* Callbacks */
 static gboolean cb_gimmix_main_window_delete_event (GtkWidget *widget, GdkEvent *event, gpointer data);
+static gboolean cb_gimmix_main_window_configure_event (GtkWidget *widget, GdkEventConfigure *event, gpointer data);
 static void	cb_play_button_clicked 	(GtkWidget *widget, gpointer data);
 static void	cb_stop_button_clicked 	(GtkWidget *widget, gpointer data);
 static void	cb_next_button_clicked 	(GtkWidget *widget, gpointer data);
@@ -83,19 +104,51 @@ static void 	cb_gimmix_progress_seek (GtkWidget *widget, GdkEvent *event);
 static void 	cb_volume_scale_changed (GtkWidget *widget, gpointer data);
 static void	cb_volume_slider_scroll (GtkWidget *widget, GdkEventScroll *event);
 static void 	cb_volume_button_clicked (GtkWidget *widget, gpointer data);
+static void		gimmix_reposition_volume_window (GtkWidget *volwindow);
 static gboolean cb_gimmix_key_press(GtkWidget *widget, GdkEventKey *event, gpointer userdata);
 
 /* mpd callbacks */
 static void 	gimmix_status_changed (MpdObj *mo, ChangedStatusType id);
-static void	gimmix_mpd_error (MpdObj *mo, int id, char *msg, void *userdata);
+static int	gimmix_mpd_error (MpdObj *mo, int id, char *msg, void *userdata);
+
+static void
+gimmix_update_global_song_info (void)
+{
+	mpd_Song *tsi = NULL;
+
+	tsi = mpd_playlist_get_current_song (gmo);
+	if (tsi!=NULL)
+	{
+		glob_song_info = tsi;
+	}
+	
+	return;
+}
 
 static void
 gimmix_status_changed (MpdObj *mo, ChangedStatusType id)
 {
-	if (id&MPD_CST_SONGID)
-	{	
+	gimmix_update_global_song_info ();
+	
+	if (!(id&MPD_CST_STATE) && (id&MPD_CST_SONGID || id&MPD_CST_DATABASE))
+	{
+		gimmix_update_current_playlist (mo, mpd_playlist_get_changes(mo,0));
+		#ifdef HAVE_COVER_PLUGIN
+		g_thread_create ((GThreadFunc)gimmix_covers_plugin_update_cover,
+				FALSE,
+				FALSE,
+				NULL);
+		
+		#endif
 		gimmix_set_song_info ();
-		gimmix_update_current_playlist ();
+		#ifdef HAVE_LYRICS
+		g_thread_create ((GThreadFunc)gimmix_lyrics_plugin_update_lyrics,
+				NULL,
+				FALSE,
+				NULL);
+		#endif
+		
+		return;
 	}
 
 	if (id&MPD_CST_STATE)
@@ -105,15 +158,36 @@ gimmix_status_changed (MpdObj *mo, ChangedStatusType id)
 		{
 			gtk_image_set_from_stock (GTK_IMAGE(image_play), "gtk-media-pause", GTK_ICON_SIZE_BUTTON);
 			gtk_tooltips_set_tip (play_button_tooltip, play_button, _("Pause <x or c>"), NULL);
+			
+			#ifdef HAVE_COVER_PLUGIN
+			if (!strncasecmp(cfg_get_key_value(conf,"coverart_enable"),"true",4))
+			{
+				g_thread_create ((GThreadFunc)gimmix_covers_plugin_update_cover,
+						FALSE,
+						FALSE,
+						NULL);
+			}
+			#endif
 			gimmix_set_song_info ();
+			#ifdef HAVE_LYRICS
+			g_thread_create ((GThreadFunc)gimmix_lyrics_plugin_update_lyrics ,
+				NULL,
+				FALSE,
+				NULL);
+			#endif
+			
 		}
+		else
 		if (state == MPD_PLAYER_PAUSE)
 		{
 			gtk_image_set_from_stock (GTK_IMAGE(image_play), "gtk-media-play", GTK_ICON_SIZE_BUTTON);
 			gtk_tooltips_set_tip (play_button_tooltip, play_button, _("Play <x or c>"), NULL);
 		}
+		else
 		if (state == MPD_PLAYER_STOP)
 		{
+			//g_print ("stopped\n");
+			gimmix_show_ver_info ();
 			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(progress), 0.0);
 			gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progress), _("Stopped"));
 			if (strncasecmp(cfg_get_key_value(conf, "enable_systray"), "true", 4) == 0)
@@ -121,17 +195,27 @@ gimmix_status_changed (MpdObj *mo, ChangedStatusType id)
 				gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(tooltip->progressbar), 0.0);
 				gtk_progress_bar_set_text (GTK_PROGRESS_BAR(tooltip->progressbar), _("Stopped"));
 			}
-			gimmix_show_ver_info ();
 			
+			#ifdef HAVE_COVER_PLUGIN
+			if (!strncasecmp(cfg_get_key_value(conf,"coverart_enable"),"true",4))
+			{
+				g_thread_create ((GThreadFunc)gimmix_covers_plugin_update_cover,
+						(gpointer)TRUE,
+						FALSE,
+						NULL);
+			}
+			#endif
 			gtk_image_set_from_stock (GTK_IMAGE(image_play), "gtk-media-play", GTK_ICON_SIZE_BUTTON);
 			gtk_tooltips_set_tip (play_button_tooltip, play_button, _("Play <x or c>"), NULL);
+			gimmix_update_current_playlist (mo, mpd_playlist_get_changes(mo,0));
+			return;
 		}
 		g_object_ref_sink (play_button_tooltip);
-		gimmix_update_current_playlist ();
+		gimmix_update_current_playlist (mo, mpd_playlist_get_changes(mo,0));
 	}
 	
 	if (id&MPD_CST_PLAYLIST)
-		gimmix_update_current_playlist ();
+		gimmix_update_current_playlist (mo, mpd_playlist_get_changes(mo,0));
 
 	if (id&MPD_CST_VOLUME)
 		gimmix_update_volume ();
@@ -144,33 +228,36 @@ gimmix_status_changed (MpdObj *mo, ChangedStatusType id)
 
 	return;
 }
-
-static void
+/*
+static int
 gimmix_mpd_error (MpdObj *mo, int id, char *msg, void *userdata)
 {
-	if (id&MPD_STATUS_FAILED || id&MPD_STATS_FAILED || id&MPD_SERVER_ERROR || id&MPD_FATAL_ERROR)
-	{
-		mpd_free (gmo);
-		gmo = gimmix_mpd_connect ();
-		mpd_signal_connect_status_changed (gmo, (StatusChangedCallback)gimmix_status_changed, NULL);
-		mpd_signal_connect_error (gmo, (ErrorCallback)gimmix_mpd_error, NULL);
-	}
-	
-	return;
-}
+	g_print ("error:%s\n", msg);
+	if (id & MPD_STATUS_FAILED)
+		return FALSE;
 
-static gboolean
-cb_playlist_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
+	if (id&MPD_STATS_FAILED || id&MPD_SERVER_ERROR || id&MPD_FATAL_ERROR)
+	{
+		//mpd_free (gmo);
+		//gmo = gimmix_mpd_connect ();
+		//mpd_signal_connect_status_changed (gmo, (StatusChangedCallback)gimmix_status_changed, NULL);
+		//mpd_signal_connect_error (gmo, (ErrorCallback)gimmix_mpd_error, NULL);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+*/
+static void
+gimmix_toggle_playlist_show (gboolean show)
 {
 	static int height;
 	static int width;
-	
-	if (event != NULL)
-	if (event->button != 1)
-		return FALSE;
 
-	if( !GTK_WIDGET_VISIBLE (playlist_box) )
-	{	
+	if (show)
+	{
+		gtk_widget_show (volume_hscalebox);
+		gtk_widget_hide (volume_button);
 		gtk_widget_show (GTK_WIDGET(playlist_box));
 		if (height>0 && width>0)
 			gtk_window_resize (GTK_WINDOW(main_window), width, height);
@@ -180,18 +267,35 @@ cb_playlist_button_press (GtkWidget *widget, GdkEventButton *event, gpointer dat
 	else
 	{
 		gint w;
+		gtk_widget_hide (volume_hscalebox);
+		gtk_widget_show (volume_button);
 		w = main_window->allocation.width;
 		gtk_widget_hide (GTK_WIDGET(playlist_box));
 		gtk_window_get_size (GTK_WINDOW(main_window), &width, &height);
 		gtk_window_resize (GTK_WINDOW(main_window), 1, 1);
 		gtk_window_set_resizable (GTK_WINDOW(main_window), FALSE);
 	}
+	
+	return;
+}
+
+static gboolean
+cb_playlist_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
+{	
+	if (event != NULL)
+	if (event->button != 1)
+		return FALSE;
+	
+	if (!GTK_WIDGET_VISIBLE(playlist_box))
+		gimmix_toggle_playlist_show (TRUE);
+	else
+		gimmix_toggle_playlist_show (FALSE);
 
 	return TRUE;
 }
 
 void
-gimmix_init (void)
+gimmix_interface_widgets_init (void)
 {
 	GtkWidget 		*widget;
 	GtkWidget		*progressbox;
@@ -199,6 +303,10 @@ gimmix_init (void)
 	GdkPixbuf		*app_icon;
 	gchar			*path;
 	
+	if (inited)
+	{
+		return;
+	}
 	/* Set the application icon */
 	main_window = glade_xml_get_widget (xml, "main_window");
 	g_signal_connect (G_OBJECT(main_window), "delete-event", G_CALLBACK(cb_gimmix_main_window_delete_event), NULL);
@@ -212,7 +320,9 @@ gimmix_init (void)
 	g_free (path);
 	
 	/* connect the key press signal */
-	g_signal_connect(G_OBJECT(main_window), "key-press-event", G_CALLBACK(cb_gimmix_key_press), NULL);
+	g_signal_connect (G_OBJECT(main_window), "key-press-event", G_CALLBACK(cb_gimmix_key_press), NULL);
+	
+	g_signal_connect (G_OBJECT(main_window), "configure-event", G_CALLBACK(cb_gimmix_main_window_configure_event), (gpointer)glade_xml_get_widget(xml, "volume_window"));
 	
 	/* connect the destroy signal */
 	g_signal_connect(G_OBJECT(main_window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
@@ -235,7 +345,10 @@ gimmix_init (void)
 	
 	shuffle_toggle_button = glade_xml_get_widget (xml, "shuffle_toggle");
 	repeat_toggle_button = glade_xml_get_widget (xml, "repeat_toggle");
+	volume_window = glade_xml_get_widget (xml, "volume_window");
 	volume_scale = glade_xml_get_widget (xml, "volume_scale");
+	volume_hscalebox = glade_xml_get_widget (xml, "volume_hscalebox");
+	volume_hscale = glade_xml_get_widget (xml, "volume_hscale");
 	volume_button = glade_xml_get_widget (xml, "volume_button");
 	playlist_button = glade_xml_get_widget (xml, "playlist_button");
 	playlist_box = glade_xml_get_widget (xml, "playlistbox");
@@ -244,7 +357,13 @@ gimmix_init (void)
 	search_entry = glade_xml_get_widget (xml, "search_label");
 	play_button = glade_xml_get_widget (xml, "play_button");
 	image_play = glade_xml_get_widget (xml, "image_play");
-	
+	plcontrolshbox = glade_xml_get_widget (xml, "plcontrolshbox");
+	next_button = glade_xml_get_widget (xml, "next_button");
+	prev_button = glade_xml_get_widget (xml, "prev_button");
+	pref_button = glade_xml_get_widget (xml, "pref_button");
+	info_button = glade_xml_get_widget (xml, "info_button");
+	stop_button = glade_xml_get_widget (xml, "stop_button");
+
 	g_signal_connect (G_OBJECT(playlist_button), "button-press-event", G_CALLBACK(cb_playlist_button_press), NULL);
 
 	if (is_gimmix_repeat (gmo))
@@ -260,9 +379,11 @@ gimmix_init (void)
 
 	g_signal_connect (G_OBJECT(volume_scale), "value_changed", G_CALLBACK(cb_volume_scale_changed), NULL);
 	g_signal_connect (G_OBJECT(volume_scale), "scroll_event", G_CALLBACK(cb_volume_slider_scroll), NULL);
+	g_signal_connect (G_OBJECT(volume_hscale), "scroll_event", G_CALLBACK(cb_volume_slider_scroll), NULL);
 	vol_adj = gtk_range_get_adjustment (GTK_RANGE(volume_scale));
+	gtk_range_set_adjustment (GTK_RANGE(volume_hscale), vol_adj);
 	gtk_adjustment_set_value (GTK_ADJUSTMENT(vol_adj), mpd_status_get_volume (gmo));
-	g_signal_connect (G_OBJECT(volume_button), "clicked", G_CALLBACK(cb_volume_button_clicked), (gpointer)glade_xml_get_widget(xml, "volume_window"));
+	g_signal_connect (G_OBJECT(volume_button), "clicked", G_CALLBACK(cb_volume_button_clicked), volume_window);
 	g_signal_connect (G_OBJECT(volume_button), "scroll_event", G_CALLBACK(cb_volume_slider_scroll), NULL);
 
 	progress = glade_xml_get_widget (xml,"progress");
@@ -276,68 +397,183 @@ gimmix_init (void)
 		gimmix_create_systray_icon ();
 	}
 	if (strncasecmp(cfg_get_key_value(conf, "full_view_mode"), "true", 4) == 0)
-	{	
+	{
 		gtk_widget_show (playlist_box);
-		gtk_window_set_resizable (GTK_WINDOW(main_window), TRUE);
+		gimmix_toggle_playlist_show (TRUE);
 	}
 	else
-	{	
+	{
 		gtk_widget_hide (playlist_box);
-		gtk_window_set_resizable (GTK_WINDOW(main_window), FALSE);
+		gimmix_toggle_playlist_show (FALSE);
 	}
 	
-	mpd_status_update (gmo);
-	status = mpd_player_get_state (gmo);
-
-	if (status == MPD_PLAYER_PLAY)
-	{
-		gimmix_set_song_info ();
-		status = -1;
-		gtk_image_set_from_stock (GTK_IMAGE(image_play), "gtk-media-pause", GTK_ICON_SIZE_BUTTON);
-		gtk_tooltips_set_tip (play_button_tooltip, play_button, _("Pause <x or c>"), NULL);
-	}
-	else if (status == MPD_PLAYER_PAUSE)
-	{
-		gimmix_set_song_info ();
-	}
-	else if (status == MPD_PLAYER_STOP)
-	{
-		gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progress), _("Stopped"));
-		if (strncasecmp(cfg_get_key_value(conf, "enable_systray"), "true", 4) == 0)
-		gtk_progress_bar_set_text (GTK_PROGRESS_BAR(tooltip->progressbar), _("Stopped"));
-		gimmix_show_ver_info ();
-	}
-
-	g_timeout_add (300, (GSourceFunc)gimmix_timer, NULL);
-
-	/* connect the main mpd callbacks */
-	mpd_signal_connect_status_changed (gmo, (StatusChangedCallback)gimmix_status_changed, NULL);
-	mpd_signal_connect_error (gmo, (ErrorCallback)gimmix_mpd_error, NULL);
+	/* initialize metadata */
+	gimmix_metadata_init ();
 	
-	/* initialize playlist and tag editor */
-	gimmix_playlist_init ();
-	gimmix_tag_editor_init ();
-	gimmix_update_current_playlist ();
+	#ifdef HAVE_COVER_PLUGIN
+	gimmix_covers_plugin_init ();
+	#else
+	widget = glade_xml_get_widget (xml, "gimmix_plc_image_frame");
+	gtk_widget_hide (widget);
+	#endif
+	
+	#ifdef HAVE_LYRICS
+	gimmix_lyrics_plugin_init ();
+	#else
+	widget = glade_xml_get_widget (xml, "lyrics_container");
+	gtk_widget_hide (widget);
+	#endif
+	
+	widget = glade_xml_get_widget (xml, "metadata_container");
+	#ifdef HAVE_COVER_PLUGIN
+	gtk_widget_show (widget);
+	#elif defined(HAVE_LYRICS)
+	gtk_widget_show (widget);
+	#else
+	gtk_widget_hide (widget);
+	#endif
 	
 	/* initialize preferences dialog */
 	gimmix_prefs_init ();
 	
-	g_object_unref (xml);
+	/* playlist widgets */
+	gimmix_playlist_widgets_init ();
+	
+	/* tag editor widgets */
+	gimmix_tag_editor_widgets_init ();
+	
+	/* show version info */
+	gimmix_show_ver_info ();
 	
 	/* show the main window */
 	gtk_widget_show (main_window);
 	
-	/* check if library needs to be updated on startup */
-	if (strncasecmp(cfg_get_key_value(conf, "update_on_startup"), "true", 4) == 0)
-	gimmix_library_update ();
+	#ifdef HAVE_COVER_PLUGIN
+	if (!strncasecmp(cfg_get_key_value(conf,"coverart_enable"),"true",4))
+	{
+		g_thread_create ((GThreadFunc)gimmix_covers_plugin_update_cover,
+				(gpointer)TRUE,
+				FALSE,
+				NULL);
+	}
+	#endif
+}
+
+void
+gimmix_interface_disable_controls (void)
+{
+	gtk_widget_set_sensitive (repeat_toggle_button, FALSE);
+	gtk_widget_set_sensitive (shuffle_toggle_button, FALSE);
+	gtk_widget_set_sensitive (play_button, FALSE);
+	gtk_widget_set_sensitive (stop_button, FALSE);
+	gtk_widget_set_sensitive (next_button, FALSE);
+	gtk_widget_set_sensitive (prev_button, FALSE);
+	gtk_widget_set_sensitive (info_button, FALSE);
+	gtk_widget_set_sensitive (pref_button, FALSE);
+	gtk_widget_set_sensitive (volume_button, FALSE);
+	gtk_widget_set_sensitive (volume_hscalebox, FALSE);
+	
+	gimmix_playlist_disable_controls ();
+	gimmix_metadata_disable_controls ();
+}
+
+void
+gimmix_interface_enable_controls (void)
+{
+	gtk_widget_set_sensitive (repeat_toggle_button, TRUE);
+	gtk_widget_set_sensitive (shuffle_toggle_button, TRUE);
+	gtk_widget_set_sensitive (play_button, TRUE);
+	gtk_widget_set_sensitive (stop_button, TRUE);
+	gtk_widget_set_sensitive (next_button, TRUE);
+	gtk_widget_set_sensitive (prev_button, TRUE);
+	gtk_widget_set_sensitive (info_button, TRUE);
+	gtk_widget_set_sensitive (pref_button, TRUE);
+	gtk_widget_set_sensitive (volume_button, TRUE);
+	gtk_widget_set_sensitive (volume_hscalebox, TRUE);
+	
+	gimmix_playlist_enable_controls ();
+	gimmix_metadata_enable_controls ();
+}
+
+void
+gimmix_init (void)
+{
+	if (!inited)
+	{
+		inited = TRUE;
+		mpd_status_update (gmo);
+		status = mpd_player_get_state (gmo);
+		gimmix_update_global_song_info ();
+		if (status == MPD_PLAYER_PLAY)
+		{
+			gimmix_set_song_info ();
+			status = -1;
+			gtk_image_set_from_stock (GTK_IMAGE(image_play), "gtk-media-pause", GTK_ICON_SIZE_BUTTON);
+			gtk_tooltips_set_tip (play_button_tooltip, play_button, _("Pause <x or c>"), NULL);
+		}
+		else if (status == MPD_PLAYER_PAUSE)
+		{
+			gimmix_set_song_info ();
+		}
+		else if (status == MPD_PLAYER_STOP)
+		{
+			gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progress), _("Stopped"));
+			if (!strncasecmp(cfg_get_key_value(conf, "enable_systray"),"true",4))
+			gtk_progress_bar_set_text (GTK_PROGRESS_BAR(tooltip->progressbar), _("Stopped"));
+			gimmix_show_ver_info ();
+		}
+		gtk_adjustment_set_value (GTK_ADJUSTMENT(gtk_range_get_adjustment(GTK_RANGE(volume_scale))), mpd_status_get_volume (gmo));
+		gimmix_playlist_init ();
+		/* check if library needs to be updated on startup */
+		if (strncasecmp(cfg_get_key_value(conf, "update_on_startup"), "true", 4) == 0)
+			gimmix_library_update ();
+		
+		g_object_unref (xml);
+	}
+	
+	g_timeout_add (300, (GSourceFunc)gimmix_timer, NULL);
+	
+	/* update current playlist */
+	gimmix_update_current_playlist (gmo, mpd_playlist_get_changes(gmo,0));
+
+	/* set song info */
+	status = mpd_player_get_state (gmo);
+	if (status == MPD_PLAYER_PLAY || status == MPD_PLAYER_PAUSE)
+	{
+		gimmix_set_song_info ();
+	}
+	
+	#ifdef HAVE_COVER_PLUGIN
+	if (!strncasecmp(cfg_get_key_value(conf,"coverart_enable"),"true",4))
+	{
+		g_thread_create ((GThreadFunc)gimmix_covers_plugin_update_cover,
+				FALSE,
+				FALSE,
+				NULL);
+	}
+	
+	#endif
+	#ifdef HAVE_LYRICS
+	if (status == MPD_PLAYER_PLAY || status == MPD_PLAYER_PAUSE)
+	{
+		g_thread_create ((GThreadFunc)gimmix_lyrics_plugin_update_lyrics,
+				NULL,
+				FALSE,
+				NULL);
+	}
+	#endif
+
+	/* connect the main mpd callbacks */
+	mpd_signal_connect_status_changed (gmo, (StatusChangedCallback)gimmix_status_changed, NULL);
+	//mpd_signal_connect_error (gmo, (ErrorCallback)gimmix_mpd_error, NULL);
 	
 	return;
 }
 
 static gboolean
 cb_gimmix_key_press (GtkWidget   *widget,
-					GdkEventKey *event,
-					gpointer     userdata)
+		GdkEventKey *event,
+		gpointer     userdata)
 {
 	gboolean result = FALSE;
 	gint state;
@@ -398,26 +634,52 @@ cb_gimmix_key_press (GtkWidget   *widget,
 static gboolean
 gimmix_timer (void)
 {
-	gchar 	time[15];
+	gchar 	time[32] = "";
 	int 	new_status;
 	float 	fraction;
+	if (mpd_check_connected(gmo)==FALSE)
+	{
+		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(progress), 0.0);
+		gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progress), "");
+		gimmix_show_ver_info ();
+		#ifdef HAVE_COVER_PLUGIN
+		if (!strncasecmp(cfg_get_key_value(conf,"coverart_enable"),"true",4))
+		{
+			g_thread_create ((GThreadFunc)gimmix_covers_plugin_update_cover,
+					(gpointer)TRUE, /* set default cover */
+					FALSE,
+					NULL);
+		}
+		#endif
+		return FALSE;
+	}
+	if (mpd_check_connected(gmo))
+	{
+		mpd_status_update (gmo);
+		new_status = mpd_player_get_state (gmo);
+	}
+	else
+	{
+		return FALSE;
+	}
 	
-	mpd_status_update (gmo);
-	new_status = mpd_player_get_state (gmo);
 	if (status == new_status)
 	{
 		if (status == MPD_PLAYER_PLAY || status == MPD_PLAYER_PAUSE)
 		{
 			gimmix_get_progress_status (gmo, &fraction, time);
-			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(progress), fraction);
+			if (fraction >= 0.0 && fraction <= 1.0)
+				gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(progress), fraction);
 			gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progress), time);
 			
 			/* Update the system tray tooltip progress bar */
-			if (strncasecmp(cfg_get_key_value(conf, "enable_notification"), "true", 4) == 0)
-			{
-				gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(tooltip->progressbar), fraction);
-				gtk_progress_bar_set_text (GTK_PROGRESS_BAR(tooltip->progressbar), time);
-			}
+			if (strncasecmp(cfg_get_key_value(conf, "enable_systray"), "true", 4) == 0)
+				if (strncasecmp(cfg_get_key_value(conf, "enable_notification"), "true", 4) == 0)
+				{
+					if (fraction >= 0.0 && fraction <= 1.0)
+						gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(tooltip->progressbar), fraction);
+					gtk_progress_bar_set_text (GTK_PROGRESS_BAR(tooltip->progressbar), time);
+				}
 		}
 		return TRUE;
 	}
@@ -462,23 +724,30 @@ cb_stop_button_clicked (GtkWidget *widget, gpointer data)
 static void
 cb_volume_button_clicked (GtkWidget *widget, gpointer data)
 {
-	gint x, y;
-	
 	if (GTK_WIDGET_VISIBLE (data))
 		gtk_widget_hide (data);
 	else
-	{	
-		gdk_window_get_origin (widget->window, &x, &y);
-		x += (widget->allocation.x);
-		y += (widget->allocation.y + widget->allocation.height);
-		gtk_window_resize (GTK_WINDOW(data), widget->allocation.width, 1);
-		gtk_window_move (GTK_WINDOW(data), x, y);
 		gtk_widget_show (data);
-	}
-	
+		gimmix_reposition_volume_window (data);
 	return;
 }
 
+static void
+gimmix_reposition_volume_window (GtkWidget *volwindow)
+{
+		if (volwindow == NULL || !GTK_WIDGET_VISIBLE(volwindow))
+		return;
+		
+		gint x, y;
+		gdk_window_get_origin (volume_button->window, &x, &y);
+		x += (volume_button->allocation.x);
+		y += (volume_button->allocation.y + volume_button->allocation.height);
+		gtk_window_resize (GTK_WINDOW(volwindow), volume_button->allocation.width, 1);
+		gtk_window_move (GTK_WINDOW(volwindow), x, y);
+		
+		return;
+}
+
 static void
 cb_repeat_button_toggled (GtkToggleButton *button, gpointer data)
 {
@@ -600,7 +869,7 @@ gimmix_update_shuffle (void)
 static void
 gimmix_update_volume ()
 {
-	gint 			volume;
+	gint 		volume;
 	GtkAdjustment	*volume_adj;
 	
 	volume_adj = gtk_range_get_adjustment (GTK_RANGE(volume_scale));
@@ -643,6 +912,8 @@ gimmix_window_visible_toggle (void)
 	else
 	{	
 		gtk_window_get_position (GTK_WINDOW(main_window), &x, &y);
+		/* hide the volume window if not hidden */
+		gtk_widget_hide (GTK_WIDGET(volume_window));
 		gtk_widget_hide (GTK_WIDGET(main_window));
 	}
 
@@ -664,24 +935,28 @@ gimmix_set_song_info (void)
 {
 	gchar 		*markup;
 	gchar 		*title;
-	SongInfo 	*song = NULL;
+	mpd_Song	*song = NULL;
+	
+	song = mpd_playlist_get_current_song (gmo);
 	
-	song = gimmix_get_song_info (gmo);
 	if (song == NULL)
-	return;
+	{
+		g_print ("song = NULL\n");
+		return;
+	}
 	
 	if (song->title != NULL)
 	{
 		title = g_strdup_printf ("%s - %s", "Gimmix", song->title);
 		gtk_window_set_title (GTK_WINDOW(main_window), title);
 		g_free (title);
-		markup = g_markup_printf_escaped ("<span size=\"large\"weight=\"bold\">%s</span>", song->title);
+		markup = g_markup_printf_escaped ("<span size=\"x-large\"weight=\"bold\">%s</span>", song->title);
 	}
 	else
 	{
 		title = g_path_get_basename (song->file);
 		gimmix_strip_file_ext (title);
-		markup = g_markup_printf_escaped ("<span size=\"large\"weight=\"bold\">%s</span>", title);
+		markup = g_markup_printf_escaped ("<span size=\"x-large\"weight=\"bold\">%s</span>", title);
 		g_free (title);
 		gtk_window_set_title (GTK_WINDOW(main_window), "Gimmix");
 	}
@@ -700,7 +975,6 @@ gimmix_set_song_info (void)
 
 	if (strncasecmp(cfg_get_key_value(conf, "enable_systray"), "true", 4) == 0)
 		gimmix_update_systray_tooltip (song);
-	gimmix_free_song_info (song);
 	
 	return;
 }
@@ -712,9 +986,9 @@ gimmix_show_ver_info (void)
 	gchar 		*appver;
 
 	appver = g_strdup_printf ("%s %s", APPNAME, VERSION);
-	markup = g_markup_printf_escaped ("<span size=\"large\"weight=\"bold\">%s</span>", appver);
+	markup = g_markup_printf_escaped ("<span size=\"x-large\"weight=\"bold\">%s</span>", appver);
 	gtk_label_set_markup (GTK_LABEL(song_label), markup);
-	gtk_label_set_text (GTK_LABEL(artist_label), APPURL);
+	gtk_label_set_text (GTK_LABEL(artist_label), "");
 	gtk_window_set_title (GTK_WINDOW(main_window), APPNAME);
 	gimmix_update_systray_tooltip (NULL);
 	g_free (markup);
@@ -742,6 +1016,16 @@ cb_gimmix_main_window_delete_event (GtkWidget *widget, GdkEvent *event, gpointer
 	return FALSE;
 }
 
+static gboolean cb_gimmix_main_window_configure_event (GtkWidget *widget, GdkEventConfigure *event, gpointer data)
+{
+	if (!GTK_WIDGET_VISIBLE(data))
+		return FALSE;
+	
+	gimmix_reposition_volume_window (data);
+	return FALSE;
+}
+
+
 void
 gimmix_save_window_pos (void)
 {
@@ -781,6 +1065,10 @@ gimmix_interface_cleanup (void)
 	/* destroy system tray icon */
 	gimmix_destroy_systray_icon ();
 	
+	#if HAVE_COVERS_PLUGIN
+	gimmix_covers_plugin_cleanup ();
+	#endif
+
 	return;
 }
 
