@@ -1,7 +1,7 @@
 /*
  * gimmix-playlist.c
  *
- * Copyright (C) 2006-2007 Priyank Gosalia
+ * Copyright (C) 2006-2008 Priyank Gosalia
  *
  * Gimmix is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -45,21 +45,39 @@ GtkWidget 	*search_combo;
 GtkWidget	*search_entry;
 GtkWidget	*search_box;
 	
-static GtkWidget		*gimmix_statusbar;
-static GtkWidget		*gimmix_statusbox;
-static GtkWidget		*button_update;
-static GtkWidget		*current_playlist_treeview;
-static GtkWidget		*library_treeview;
+static GtkWidget	*gimmix_statusbar;
+static GtkWidget	*gimmix_statusbox;
+static GtkWidget	*button_update;
+static GtkWidget	*current_playlist_treeview;
+static GtkWidget	*library_treeview;
 GtkWidget		*playlists_treeview;
 GtkTreeSelection	*current_playlist_selection;
 GtkTreeSelection	*library_selection;
+GtkWidget		*library_window;
 GtkWidget		*current_pl_window;
 GtkWidget		*pls_playlist_window;
+GtkWidget		*pls_button_add;
+GtkWidget		*pls_button_del;
+GtkWidget		*pls_button_clear;
+GtkWidget		*pls_button_add;
+GtkWidget		*pls_button_save;
+GtkWidget		*pls_playlists_box;
 
 extern GtkWidget	*tag_editor_window;
 
 gchar			*loaded_playlist;
 
+void
+onDragDataRecived(GtkWidget *widget,
+		  GdkDragContext *context,
+		  int x, int y,
+		  GtkSelectionData *seldata,
+		  guint info, guint time,
+		  gpointer userdata)
+{
+	g_print ("Dropped.\n");
+}
+
 static void		gimmix_search_init (void);
 static void		gimmix_library_search (gint, gchar *);
 static void		gimmix_library_and_playlists_populate (void);
@@ -68,8 +86,8 @@ static void		gimmix_current_playlist_popup_menu (void);
 static void		gimmix_library_popup_menu (void);
 static void		gimmix_playlists_popup_menu (void);
 static gchar*		gimmix_path_get_parent_dir (gchar *);
-static void 		gimmix_load_playlist (gchar *);
-static void		gimmix_display_total_playlist_time (void);
+static void		gimmix_load_playlist (gchar *);
+static void		gimmix_display_total_playlist_time (MpdObj *);
 
 /* Callbacks */
 /* Current playlist callbacks */
@@ -78,10 +96,10 @@ static void		cb_current_playlist_right_click (GtkTreeView *treeview, GdkEventBut
 static void		cb_repeat_menu_toggled (GtkCheckMenuItem *item, gpointer data);
 static void		cb_shuffle_menu_toggled (GtkCheckMenuItem *item, gpointer data);
 static void		cb_current_playlist_delete_press (GtkWidget *widget, GdkEventKey *event, gpointer data);
-static void 	cb_add_button_clicked (GtkWidget *widget, gpointer data);
-static void		cb_playlists_button_toggled (GtkToggleButton *button, gpointer data);
+static void 		cb_add_button_clicked (GtkWidget *widget, gpointer data);
 
 static void		gimmix_current_playlist_remove_song (void);
+static void		gimmix_current_playlist_crop_song (void);
 static void		gimmix_current_playlist_song_info (void);
 static void		gimmix_current_playlist_clear (void);
 static gboolean		gimmix_update_player_status (gpointer data);
@@ -89,23 +107,38 @@ static gboolean		gimmix_update_player_status (gpointer data);
 /* Library browser callbacks */
 static void		cb_library_dir_activated (gpointer data);
 static void		gimmix_library_song_info (void);
-static void 		cb_playlist_activated (GtkTreeView *);
+static void		cb_playlist_activated (GtkTreeView *);
 static void		cb_library_right_click (GtkTreeView *treeview, GdkEventButton *event);
 static void		cb_library_popup_add_clicked (GtkWidget *widget, gpointer data);
+static void		cd_library_popup_replace_clicked (GtkWidget *widget, gpointer data);
 static void		cb_search_keypress (GtkWidget *widget, GdkEventKey *event, gpointer data);
 
+
 /* Playlist browser callbacks */
 static void		gimmix_update_playlists_treeview (void);
 static void		gimmix_playlist_save_dialog_show (void);
-static void 		cb_gimmix_playlist_save_response (GtkDialog *dlg, gint arg1, gpointer dialog);
+static void		cb_gimmix_playlist_save_response (GtkDialog *dlg, gint arg1, gpointer dialog);
 static void		cb_playlists_right_click (GtkTreeView *treeview, GdkEventButton *event);
 static bool		cb_all_playlist_button_press (GtkTreeView *treeview, GdkEventButton *event);
 static void		cb_gimmix_playlist_remove ();
 static void		cb_gimmix_playlist_load ();
 static void		cb_playlists_delete_press (GtkWidget *widget, GdkEventKey *event, gpointer data);
 
+static void
+gimmix_playlist_search_widgets_init (void)
+{
+	search_box = glade_xml_get_widget (xml, "search_box");
+	search_combo = glade_xml_get_widget (xml, "search_combo");
+	search_entry = glade_xml_get_widget (xml, "search_entry");
+	
+	gtk_combo_box_set_active (GTK_COMBO_BOX(search_combo), 0);
+	g_signal_connect (G_OBJECT(search_entry), "key_release_event", G_CALLBACK(cb_search_keypress), NULL);
+	
+	return;
+}
+
 void
-gimmix_playlist_init (void)
+gimmix_playlist_widgets_init (void)
 {
 	GtkTreeModel		*current_playlist_model;
 	GtkListStore		*current_playlist_store;
@@ -115,30 +148,30 @@ gimmix_playlist_init (void)
 	current_playlist_treeview = glade_xml_get_widget (xml, "current_playlist_treeview");
 	playlists_treeview = glade_xml_get_widget (xml, "playlists_treeview");
 	library_treeview = glade_xml_get_widget (xml, "album");
+	library_window = glade_xml_get_widget (xml,"library_window");
 	
-	current_playlist_column = gtk_tree_view_column_new ();
 	current_playlist_renderer = gtk_cell_renderer_text_new ();
-	gtk_tree_view_column_pack_start (current_playlist_column, current_playlist_renderer, FALSE);
-	gtk_tree_view_column_set_title (current_playlist_column, _("Title"));
+	current_playlist_column = gtk_tree_view_column_new_with_attributes (_("Title"),
+										current_playlist_renderer,
+										"markup", 0,
+										NULL);
 	gtk_tree_view_column_set_resizable (current_playlist_column, TRUE);
 	gtk_tree_view_column_set_min_width (current_playlist_column, 200);
-	gtk_tree_view_column_set_resizable (current_playlist_column, TRUE);
 	g_object_set (G_OBJECT(current_playlist_column), "expand", TRUE, "spacing", 4, NULL);
-	gtk_tree_view_column_add_attribute (current_playlist_column, current_playlist_renderer, "markup", 0);
 	gtk_tree_view_append_column (GTK_TREE_VIEW(current_playlist_treeview), current_playlist_column);
 							
 	current_playlist_renderer = gtk_cell_renderer_text_new ();
-	current_playlist_column = gtk_tree_view_column_new_with_attributes (_("Length"), current_playlist_renderer,
-			"markup", 3,
-			NULL);
-	gtk_tree_view_column_pack_end (current_playlist_column, current_playlist_renderer, FALSE);
+	current_playlist_column = gtk_tree_view_column_new_with_attributes (_("Length"),
+										current_playlist_renderer,
+										"markup", 3,
+										NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW(current_playlist_treeview), current_playlist_column);
 
 	current_playlist_store = gtk_list_store_new (4,
-						G_TYPE_STRING, 		/* name (0) */
-						G_TYPE_STRING, 		/* path (1) */
-						G_TYPE_INT,			/* id (2) */
-						G_TYPE_STRING);    	/* length (3) */
+						G_TYPE_STRING, 	/* name (0) */
+						G_TYPE_STRING, 	/* path (1) */
+						G_TYPE_INT,		/* id	(2) */
+						G_TYPE_STRING); /* length (3) */
 	current_playlist_model	= GTK_TREE_MODEL (current_playlist_store);
 	current_playlist_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(current_playlist_treeview));
 	gtk_tree_selection_set_mode (current_playlist_selection, GTK_SELECTION_MULTIPLE);
@@ -156,65 +189,120 @@ gimmix_playlist_init (void)
 	
 	/* initialize playlist toolbar */
 	/* Add button */
-	g_signal_connect (G_OBJECT(glade_xml_get_widget(xml, "button_add")), "clicked", G_CALLBACK(cb_add_button_clicked), (gpointer)glade_xml_get_widget(xml, "library_window"));
+	pls_button_add = glade_xml_get_widget (xml, "button_add");
+	g_signal_connect (G_OBJECT(pls_button_add), "clicked", G_CALLBACK(cb_add_button_clicked), (gpointer)glade_xml_get_widget(xml, "library_window"));
 	
 	/* Remove button */
-	g_signal_connect (G_OBJECT(glade_xml_get_widget(xml, "button_remove")), "clicked", G_CALLBACK(gimmix_current_playlist_remove_song), NULL);
+	pls_button_del = glade_xml_get_widget (xml, "button_remove");
+	g_signal_connect (G_OBJECT(pls_button_del), "clicked", G_CALLBACK(gimmix_current_playlist_remove_song), NULL);
 	
 	/* Clear button */
-	g_signal_connect (G_OBJECT(glade_xml_get_widget(xml, "button_clear")), "clicked", G_CALLBACK(gimmix_current_playlist_clear), NULL);
+	pls_button_clear = glade_xml_get_widget (xml, "button_clear");
+	g_signal_connect (G_OBJECT(pls_button_clear), "clicked", G_CALLBACK(gimmix_current_playlist_clear), NULL);
 	
 	/* Update button */
 	button_update = glade_xml_get_widget (xml, "button_update");
 	g_signal_connect (G_OBJECT(button_update), "clicked", G_CALLBACK(gimmix_library_update), NULL);
 	
 	/* Save button */
-	g_signal_connect (G_OBJECT (glade_xml_get_widget(xml, "button_save")), "clicked", G_CALLBACK(gimmix_playlist_save_dialog_show), NULL);
+	pls_button_save = glade_xml_get_widget(xml, "button_save");
+	g_signal_connect (G_OBJECT (pls_button_save), "clicked", G_CALLBACK(gimmix_playlist_save_dialog_show), NULL);
+	
+	pls_playlists_box =  glade_xml_get_widget (xml, "pls_playlist_window");
 	
 	/* Playlist toggle button */
 	current_pl_window = glade_xml_get_widget (xml, "current_pl_window");
 	pls_playlist_window = glade_xml_get_widget (xml, "pls_playlist_window");
-	g_signal_connect (G_OBJECT (glade_xml_get_widget(xml, "playlist_toggle")), "toggled", G_CALLBACK(cb_playlists_button_toggled), NULL);
-	
-	/* populate the file browser */
-	gimmix_library_and_playlists_populate ();
-	
-	/* Initialize playlist search */
-	gimmix_search_init ();
-	
+
+	/* Drag and Drop */
+	enum { TARGET_STRING, TARGET_URI };
+  	GtkTargetEntry targetentries[] =
+    	{
+     		{ "STRING",        0, TARGET_STRING },
+      		{ "text/plain",    0, TARGET_STRING },      		
+		{ "text/uri-list", 0, TARGET_URI },
+	};
+
+	gtk_drag_dest_set(GTK_WIDGET(current_playlist_treeview),
+		    GTK_DEST_DEFAULT_ALL,
+		    targetentries, 3,
+		    GDK_ACTION_COPY|GDK_ACTION_MOVE);
+	gtk_drag_source_set(GTK_WIDGET(current_playlist_treeview),
+		    GDK_BUTTON1_MASK,
+		    targetentries, 3,
+		    GDK_ACTION_COPY|GDK_ACTION_MOVE);
+	g_signal_connect(GTK_WIDGET(current_playlist_treeview),
+		   "drag_data_received",
+		   G_CALLBACK(onDragDataRecived),
+		   current_playlist_store);
 	loaded_playlist = NULL;
-	return;
+	
+	gimmix_playlist_search_widgets_init ();
+}
+
+void
+gimmix_playlist_disable_controls (void)
+{
+	gtk_widget_set_sensitive (pls_button_add, FALSE);
+	gtk_widget_set_sensitive (pls_button_del, FALSE);
+	gtk_widget_set_sensitive (pls_button_clear, FALSE);
+	gtk_widget_set_sensitive (button_update, FALSE);
+	gtk_widget_set_sensitive (pls_button_save, FALSE);
+	gtk_widget_set_sensitive (pls_playlists_box, FALSE);
+	gtk_widget_set_sensitive (current_playlist_treeview, FALSE);
+}
+
+void
+gimmix_playlist_enable_controls (void)
+{
+	gtk_widget_set_sensitive (pls_button_add, TRUE);
+	gtk_widget_set_sensitive (pls_button_del, TRUE);
+	gtk_widget_set_sensitive (pls_button_clear, TRUE);
+	gtk_widget_set_sensitive (button_update, TRUE);
+	gtk_widget_set_sensitive (pls_button_save, TRUE);
+	gtk_widget_set_sensitive (pls_playlists_box, TRUE);
+	gtk_widget_set_sensitive (current_playlist_treeview, TRUE);
 }
 
 static void
 gimmix_search_init (void)
 {
-	search_box = glade_xml_get_widget (xml, "search_box");
-	search_combo = glade_xml_get_widget (xml, "search_combo");
-	search_entry = glade_xml_get_widget (xml, "search_entry");
-	
-	gtk_combo_box_set_active (GTK_COMBO_BOX(search_combo), 0);
-	g_signal_connect (G_OBJECT(search_entry), "key_release_event", G_CALLBACK(cb_search_keypress), NULL);
-	
 	if (strncasecmp(cfg_get_key_value(conf, "enable_search"), "true", 4) != 0)
 		gtk_widget_hide (search_box);
-	
+		
 	return;
 }
 
 void
-gimmix_update_current_playlist (void)
+gimmix_playlist_init (void)
+{
+	/* populate the file browser */
+	gimmix_library_and_playlists_populate ();
+	
+	/* Initialize playlist search */
+	gimmix_search_init ();
+
+	return;
+}
+
+void
+gimmix_update_current_playlist (MpdObj *mo, MpdData *pdata)
 {
 	GtkListStore	*current_playlist_store;
-	GtkTreeIter		current_playlist_iter;
-	gint 			new;
-	MpdData 		*data;
-	gint			current_song_id = -1;
-
-	new = mpd_playlist_get_playlist_id (gmo);
-	data = mpd_playlist_get_changes (gmo, 0);
-	current_song_id = mpd_player_get_current_song_id (gmo);
+	GtkTreeIter	current_playlist_iter;
+	gint 		new;
+	gint		current_song_id = -1;
+	MpdData		*data = pdata;
 
+	if (mo!=NULL && mpd_check_connected(mo))
+	{
+		new = mpd_playlist_get_playlist_id (mo);
+		current_song_id = mpd_player_get_current_song_id (mo);
+	}
+	else
+	{
+		return;
+	}
 	current_playlist_store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW(current_playlist_treeview)));
 	gtk_list_store_clear (current_playlist_store);
 	
@@ -235,12 +323,12 @@ gimmix_update_current_playlist (void)
 			}
 			else
 			{
-				gchar *file = g_path_get_basename(data->song->file);
+				gchar *file = g_path_get_basename (data->song->file);
 				gimmix_strip_file_ext (file);
 				title = g_markup_printf_escaped ("<span size=\"medium\"weight=\"bold\">%s</span>", file);
 				g_free (file);
 			}
-			gimmix_get_total_time_for_song (gmo, data->song, time);
+			gimmix_get_total_time_for_song (mo, data->song, time);
 			ti = g_markup_printf_escaped ("<span size=\"medium\" weight=\"bold\">%s</span>", time);
 		}
 		else
@@ -257,7 +345,7 @@ gimmix_update_current_playlist (void)
 				title = g_markup_printf_escaped (g_path_get_basename(data->song->file));
 				gimmix_strip_file_ext (title);
 			}
-			gimmix_get_total_time_for_song (gmo, data->song, time);
+			gimmix_get_total_time_for_song (mo, data->song, time);
 			ti = NULL;
 		}
 		
@@ -275,22 +363,27 @@ gimmix_update_current_playlist (void)
 		g_free (title);
 	}
 	gtk_tree_view_set_model (GTK_TREE_VIEW (current_playlist_treeview), GTK_TREE_MODEL(current_playlist_store));
-	gimmix_display_total_playlist_time ();
-	
-	mpd_data_free (data);
-	
+	gimmix_display_total_playlist_time (mo);
+	/*
+	if (pdata)
+	{
+		mpd_data_free (pdata);
+	}
+	*/
 	return;
 }
 
 static void
-gimmix_display_total_playlist_time (void)
+gimmix_display_total_playlist_time (MpdObj *mo)
 {
-	MpdData 	*data = NULL;
 	gchar		*time_string;
 	gint		time = 0;
+	guint		len = 0;
+	MpdData		*data;
 	
-	data = mpd_playlist_get_changes (gmo, 0);
-	if (mpd_playlist_get_playlist_length(gmo) == 0)
+	len = mpd_playlist_get_playlist_length (mo);
+	data = mpd_playlist_get_changes (mo, 0);
+	if (!len)
 	{
 		gtk_widget_hide (gimmix_statusbox);
 		return;
@@ -298,22 +391,26 @@ gimmix_display_total_playlist_time (void)
 	
 	while (data != NULL)
 	{
-		time += data->song->time;
+		if (data->song)
+			time += data->song->time;
 		data = mpd_data_get_next (data);
 	}
 	
 	if (time > 0)
 	{
-		time_string = g_strdup_printf ("%d %s, %s%d %s", mpd_playlist_get_playlist_length(gmo), _("Items"), _("Total Duration: "), time/60, _("minutes"));
+		time_string = g_strdup_printf ("%d %s, %s%d %s", len, _("Items"), _("Total Duration: "), time/60, _("minutes"));
+		//g_print ("%s\n",time_string);
 		gtk_label_set_text (GTK_LABEL(gimmix_statusbar), time_string);
 		gtk_widget_show (gimmix_statusbox);
 		g_free (time_string);
 	}
+	/*
 	else
 	{
+		g_print ("called totla\n");
 		gtk_widget_hide (gimmix_statusbox);
 	}
-	
+	*/
 	return;
 }
 
@@ -336,7 +433,7 @@ gimmix_library_and_playlists_populate (void)
 
 	library_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(library_treeview));
 	gtk_tree_selection_set_mode (library_selection, GTK_SELECTION_MULTIPLE);
-	dir_renderer		= gtk_cell_renderer_pixbuf_new();
+	dir_renderer = gtk_cell_renderer_pixbuf_new();
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (library_treeview),
 							-1,
 							"Icon",
@@ -369,14 +466,14 @@ gimmix_library_and_playlists_populate (void)
 							NULL);
 	
 	dir_store 	= gtk_list_store_new (4, 
-								GDK_TYPE_PIXBUF, 	/* icon (0) */
-								G_TYPE_STRING, 		/* name (1) */
-								G_TYPE_STRING,		/* path (2) */
-								G_TYPE_INT);		/* type DIR/SONG (3) */
+						GDK_TYPE_PIXBUF, 	/* icon (0) */
+						G_TYPE_STRING, 		/* name (1) */
+						G_TYPE_STRING,		/* path (2) */
+						G_TYPE_INT);		/* type DIR/SONG (3) */
 	
 	pls_store 	= gtk_list_store_new (2, 
-								GDK_TYPE_PIXBUF, 	/* icon (0) */
-								G_TYPE_STRING);		/* name (1) */
+						GDK_TYPE_PIXBUF, 	/* icon (0) */
+						G_TYPE_STRING);		/* name (1) */
 	
 	path = gimmix_get_full_image_path (GIMMIX_MEDIA_ICON);
 	song_pixbuf = gdk_pixbuf_new_from_file_at_size (path, 16, 16, NULL);
@@ -450,14 +547,15 @@ gimmix_library_and_playlists_populate (void)
 	g_signal_connect (playlists_treeview, "row-activated", G_CALLBACK(cb_playlist_activated), NULL);
 	g_signal_connect (library_treeview, "button-press-event", G_CALLBACK(cb_all_playlist_button_press), NULL);
 	g_signal_connect (library_treeview, "button-release-event", G_CALLBACK(cb_library_right_click), NULL);
-	g_signal_connect (glade_xml_get_widget(xml,"library_window"), "delete-event", G_CALLBACK(gtk_widget_hide_on_delete), NULL);
+	g_signal_connect (library_window, "delete-event", G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
 	g_object_unref (dir_model);
 	g_object_unref (pls_model);
+	/*
 	g_object_unref (dir_pixbuf);
 	g_object_unref (song_pixbuf);
 	g_object_unref (pls_pixbuf);
-	
+	*/
 	return;
 }
 
@@ -582,7 +680,7 @@ cb_current_playlist_delete_press (GtkWidget *widget, GdkEventKey *event, gpointe
 static void
 cb_current_playlist_double_click (GtkTreeView *treeview)
 {
-	GtkTreeModel 		*model;
+	GtkTreeModel 			*model;
 	GList				*list;
 	GtkTreeIter			iter;
 	gint 				id;
@@ -628,17 +726,17 @@ cb_all_playlist_button_press (GtkTreeView *treeview, GdkEventButton *event)
 static void
 cb_library_dir_activated (gpointer data)
 {
-	GtkTreeModel 		*model;
-	GtkTreeIter 		iter;
+	GtkTreeModel 			*model;
+	GtkTreeIter 			iter;
 	GList				*list;
 	gchar				*path;
-	GimmixFileType		type;
-	MpdData				*mpddata;
-	gint				id;
+	GimmixFileType			type;
+	gboolean 			added;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (library_treeview));
 	list = gtk_tree_selection_get_selected_rows (library_selection, &model);
-	
+	added = false; 
+
 	if (gtk_tree_selection_count_selected_rows (library_selection) == 1)
 	{
 		gtk_tree_model_get_iter (model, &iter, list->data);
@@ -651,20 +749,25 @@ cb_library_dir_activated (gpointer data)
 		else if (type == SONG)
 		{
 			mpd_playlist_add (gmo, path);
+			added = true;
 		}
 		
 		g_free (path);
 	}
 	
-	mpddata = mpd_playlist_get_changes (gmo, mpd_playlist_get_playlist_id(gmo));
-	if (strncasecmp(cfg_get_key_value(conf, "play_on_add"), "true", 4) == 0 && mpddata!=NULL)
+	if ((strncasecmp(cfg_get_key_value(conf, "play_on_add"), "true", 4) == 0) && (added == true))
 	{
-		id = mpddata->song->id;
-		mpd_player_play_id (gmo, mpddata->song->id);
-		gimmix_set_song_info ();
+		/* If we're adding, there might already be a song playing. */
+		int state;
+
+		state = mpd_player_get_state (gmo);
+
+		if (state == MPD_PLAYER_PAUSE || state == MPD_PLAYER_STOP)
+		{	
+			mpd_player_play (gmo);
+		}
 	}
 	mpd_status_update (gmo);
-	mpd_data_free (mpddata);
 		
 	/* free the list */
 	g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
@@ -678,11 +781,9 @@ cb_library_popup_add_clicked (GtkWidget *widget, gpointer data)
 {
 	GtkTreeModel 		*model;
 	GtkTreeIter 		iter;
-	GList				*list;
-	gchar				*path;
+	GList			*list;
+	gchar			*path;
 	GimmixFileType		type;
-	MpdData				*mpddata;
-	gint				id;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (library_treeview));
 	list = gtk_tree_selection_get_selected_rows (library_selection, &model);
@@ -726,15 +827,87 @@ cb_library_popup_add_clicked (GtkWidget *widget, gpointer data)
 	}
 	
 	mpd_playlist_queue_commit (gmo);
-	mpddata = mpd_playlist_get_changes (gmo, mpd_playlist_get_playlist_id(gmo));
 	if (strncasecmp(cfg_get_key_value(conf, "play_on_add"), "true", 4) == 0)
 	{
-		id = mpddata->song->id;
-		mpd_player_play_id (gmo, mpddata->song->id);
-		gimmix_set_song_info ();
+		/* If we're adding, there might already be a song playing. */
+		int state;
+
+		state = mpd_player_get_state (gmo);
+
+		if (state == MPD_PLAYER_PAUSE || state == MPD_PLAYER_STOP)
+		{	
+			mpd_player_play (gmo);
+		}
+	}
+	mpd_status_update (gmo);
+		
+	/* free the list */
+	g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (list);
+	
+	return;
+}
+
+static void cd_library_popup_replace_clicked (GtkWidget *widget, gpointer data)
+{
+
+	GtkTreeModel 		*model;
+	GtkTreeIter 		iter;
+	GList			*list;
+	gchar			*path;
+	GimmixFileType		type;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (library_treeview));
+	list = gtk_tree_selection_get_selected_rows (library_selection, &model);
+
+	gimmix_current_playlist_clear();	
+	mpd_status_update (gmo);
+
+	if (gtk_tree_selection_count_selected_rows (library_selection) == 1)
+	{
+		gtk_tree_model_get_iter (model, &iter, list->data);
+		gtk_tree_model_get (model, &iter, 2, &path, 3, &type, -1);
+		
+		if (type == DIR)
+		{
+			mpd_playlist_queue_add (gmo, path);
+		}
+		else if (type == SONG)
+		{
+			mpd_playlist_add (gmo, path);
+		}
+		
+		g_free (path);
+	}
+	else
+	while (list != NULL)
+	{
+		gtk_tree_model_get_iter (model, &iter, list->data);
+		gtk_tree_model_get (model, &iter, 2, &path, 3, &type, -1);
+	
+		
+		if (type == DIR)
+		{	
+			mpd_playlist_queue_add (gmo, path);
+			g_free (path);
+		}
+		
+		if (type == SONG)
+		{
+			mpd_playlist_queue_add (gmo, path);
+			g_free (path);
+		}
+		
+		list = g_list_next (list);
+	}
+
+	mpd_playlist_queue_commit (gmo);
+	
+	if (strncasecmp(cfg_get_key_value(conf, "play_on_add"), "true", 4) == 0)
+	{
+		gimmix_play(gmo);
 	}
 	mpd_status_update (gmo);
-	mpd_data_free (mpddata);
 		
 	/* free the list */
 	g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
@@ -749,7 +922,7 @@ cb_playlist_activated (GtkTreeView *treeview)
 	GtkTreeModel 		*model;
 	GtkTreeSelection 	*selection;
 	GtkTreeIter 		iter;
-	gchar 				*pl_name;
+	gchar 			*pl_name;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
@@ -763,6 +936,8 @@ cb_playlist_activated (GtkTreeView *treeview)
 	
 	gimmix_load_playlist (pl_name);
 	g_free (pl_name);
+
+	return;
 }
 
 static void
@@ -849,7 +1024,7 @@ gimmix_update_library_with_dir (gchar *dir)
 	song_pixbuf = gdk_pixbuf_new_from_file_at_size (path, 16, 16, NULL);
 	g_free (path);
 	
-	if (dir != "/")
+	if (strcmp(dir,"/"))
 	{	
 		parent = gimmix_path_get_parent_dir (dir);
 		gtk_list_store_append (dir_store, &dir_iter);
@@ -1052,6 +1227,51 @@ gimmix_current_playlist_remove_song (void)
 	return;
 }	
 
+static void
+gimmix_current_playlist_crop_song (void)
+{
+	GtkTreeModel			*current_playlist_model;
+	GtkTreeIter			iter;
+	GList				*list;
+	GList				*all;
+	gint				id;
+
+	current_playlist_model = gtk_tree_view_get_model (GTK_TREE_VIEW(current_playlist_treeview));
+	list = gtk_tree_selection_get_selected_rows (current_playlist_selection, &current_playlist_model);
+	gtk_tree_selection_select_all (current_playlist_selection);
+
+	while (list != NULL)
+	{
+		if (gtk_tree_model_get_iter (current_playlist_model, &iter, list->data))
+		{
+			gtk_tree_selection_unselect_iter (current_playlist_selection, &iter);
+		}
+		list = g_list_next (list);
+	}
+
+	all = gtk_tree_selection_get_selected_rows (current_playlist_selection, &current_playlist_model);
+	while (all != NULL)
+	{
+		if (TRUE == gtk_tree_model_get_iter (current_playlist_model, &iter, all->data))
+		{
+			gtk_tree_model_get (current_playlist_model, &iter, 2, &id, -1);
+			mpd_playlist_queue_delete_id (gmo, id);
+		}
+		all = g_list_next (all);
+	}
+	mpd_playlist_queue_commit (gmo);
+	mpd_status_update (gmo);
+	
+	/* free the list */
+	g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (list);
+	g_list_foreach (all, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (all);
+	
+	return;
+}	
+
+
 static void
 gimmix_current_playlist_clear (void)
 {
@@ -1075,15 +1295,22 @@ gimmix_current_playlist_clear (void)
 static void
 gimmix_current_playlist_popup_menu (void)
 {
-	GtkWidget *menu, *menu_item;
+	GtkWidget *menu, *menu_item, *image;
 
 	menu = gtk_menu_new ();
+	image = gtk_image_new_from_stock ("gtk-cut", GTK_ICON_SIZE_MENU); 
 
 	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
 	g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK (gimmix_current_playlist_remove_song), NULL);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 	gtk_widget_show (menu_item);
 	
+	menu_item = gtk_image_menu_item_new_with_label (_("Crop"));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menu_item), image);
+	g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK (gimmix_current_playlist_crop_song), NULL);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	gtk_widget_show (menu_item);
+
 	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_INFO, NULL);
 	g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK (gimmix_current_playlist_song_info), NULL);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
@@ -1144,15 +1371,23 @@ gimmix_library_popup_menu (void)
 	GtkWidget 			*menu;
 	GtkWidget 			*menu_item;
 	GtkWidget			*image;
+	GtkWidget			*replace_icon;
 
 	menu = gtk_menu_new ();
 	image = gtk_image_new_from_stock ("gtk-refresh", GTK_ICON_SIZE_MENU);
+	replace_icon = gtk_image_new_from_stock ("gtk-convert", GTK_ICON_SIZE_MENU);
 	
 	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ADD, NULL);
 	g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK (cb_library_popup_add_clicked), (gpointer)1);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 	gtk_widget_show (menu_item);
 	
+	menu_item = gtk_image_menu_item_new_with_label (_("Replace"));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menu_item), replace_icon);
+	g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK (cd_library_popup_replace_clicked), NULL);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	gtk_widget_show (menu_item);
+
 	if (gtk_tree_selection_count_selected_rows(library_selection) == 1)
 	{
 		menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_INFO, NULL);
@@ -1269,7 +1504,7 @@ gimmix_update_player_status (gpointer data)
 	if (mpd_status_db_is_updating (gmo))
 		return TRUE;
 		
-	gimmix_display_total_playlist_time ();		
+	gimmix_display_total_playlist_time (NULL);		
 	gimmix_update_library_with_dir ("/");
 	/* re-enable the update button on the toolbar */
 	gtk_widget_set_sensitive (button_update, TRUE);	
@@ -1413,15 +1648,14 @@ cb_gimmix_playlist_load ()
 		gtk_tree_model_get (pls_treemodel, &iter,
 							1, &path,
 							-1);
+		mpd_playlist_queue_load (gmo, path);
+		gimmix_current_playlist_clear ();
+		mpd_playlist_queue_commit (gmo);
+	
+		gimmix_load_playlist (path);
+		g_free (path);
 	}
 	
-	mpd_playlist_queue_load (gmo, path);
-	gimmix_current_playlist_clear ();
-	mpd_playlist_queue_commit (gmo);
-	
-	gimmix_load_playlist (path);
-	g_free (path);
-	
 	return;
 }
 
@@ -1446,21 +1680,3 @@ cb_add_button_clicked (GtkWidget *widget, gpointer data)
 	return;	
 }
 
-static void
-cb_playlists_button_toggled (GtkToggleButton *button, gpointer data)
-{
-	if (GTK_WIDGET_VISIBLE(current_pl_window))
-	{	
-		gtk_widget_hide (GTK_WIDGET(current_pl_window));
-		gtk_widget_show (GTK_WIDGET(pls_playlist_window));
-	}
-	else
-	{
-		gtk_widget_show (GTK_WIDGET(current_pl_window));
-		gtk_widget_hide (GTK_WIDGET(pls_playlist_window));
-	}
-	
-	return;	
-}
-
-
