@@ -9,11 +9,19 @@
 #include <libmpd/libmpdclient.h>
 #include <gtk/gtk.h>
 
-/* Initialize the playlist interface, file browser and setup signals */
+/* Initialize the playlist widgets and setup signals */
+void gimmix_playlist_widgets_init (void);
+
+/* Disable widgets */
+void gimmix_playlist_disable_controls (void);
+
+void gimmix_playlist_enable_controls (void);
+
+/* Initialize and populate playlists */
 void gimmix_playlist_init (void);
 
 /* update current playlist depending on the mpd playlist */
-void gimmix_update_current_playlist (void);
+void gimmix_update_current_playlist (MpdObj *mo, MpdData *pdata);
 
 /* update library */
 void gimmix_library_update (void);
